@@ -1,12 +1,12 @@
 ;;; vertico-buffer.el --- Display Vertico like a regular buffer -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021-2025 Free Software Foundation, Inc.
+;; Copyright (C) 2021-2026 Free Software Foundation, Inc.
 
 ;; Author: Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2021
-;; Version: 2.3
-;; Package-Requires: ((emacs "28.1") (compat "30") (vertico "2.3"))
+;; Version: 2.6
+;; Package-Requires: ((emacs "29.1") (compat "30") (vertico "2.6"))
 ;; URL: https://github.com/minad/vertico
 
 ;; This file is part of GNU Emacs.
@@ -92,15 +92,19 @@
 
 (defvar-local vertico-buffer--restore nil)
 
-(defun vertico-buffer--redisplay (win)
-  "Redisplay window WIN."
-  (when-let ((mbwin (active-minibuffer-window))
-             ((eq (window-buffer mbwin) (current-buffer))))
-    (unless (eq win mbwin)
-      (setq-local truncate-lines (< (window-point win)
-                                    (* 0.8 (window-width win))))
-      (set-window-point win (point))
-      (set-window-hscroll win 0))
+(defun vertico-buffer--redisplay (_)
+  "Redisplay buffer window."
+  (when-let* ((mbwin (active-minibuffer-window))
+              ((eq (window-buffer mbwin) (current-buffer)))
+              ((overlayp vertico--candidates-ov))
+              (win (overlay-get vertico--candidates-ov 'window)))
+    (setq-local truncate-lines (< (window-point win)
+                                  (* 0.8 (window-width win)))
+                vertico-count (- (/ (window-pixel-height win)
+                                    (default-line-height))
+                                 (if mode-line-format 2 1)))
+    (set-window-point win (point))
+    (set-window-hscroll win 0)
     (when vertico-buffer-hide-prompt
       (window-resize mbwin (- (window-pixel-height mbwin)) nil nil 'pixelwise)
       (set-window-vscroll mbwin 3))
@@ -153,7 +157,8 @@
                                       (let ((depth (recursion-depth)))
                                         (if (< depth 2) "" (format " [%s]" depth))))))
                      vertico-count (- (/ (window-pixel-height win)
-                                         (default-line-height)) 2))))
+                                         (default-line-height))
+                                      (if mode-line-format 2 1)))))
     (set-window-parameter win 'no-other-window t)
     (set-window-parameter win 'no-delete-other-windows t)
     (set-window-dedicated-p win t)
@@ -198,7 +203,7 @@
   "Display Vertico like a regular buffer in a large window."
   :global t :group 'vertico
   ;; Shrink current minibuffer window
-  (when-let ((win (active-minibuffer-window)))
+  (when-let* ((win (active-minibuffer-window)))
     (unless (frame-root-window-p win)
       (window-resize win (- (window-pixel-height win)) nil nil 'pixelwise))
     (with-selected-window win
@@ -208,7 +213,7 @@
        ((and (not vertico-buffer-mode) vertico-buffer--restore)
         (funcall vertico-buffer--restore))))))
 
-(cl-defmethod vertico--resize (&context (vertico-buffer-mode (eql t))))
+(cl-defmethod vertico--resize-window (_height &context (vertico-buffer-mode (eql t))))
 
 (cl-defmethod vertico--setup :after (&context (vertico-buffer-mode (eql t)))
   (vertico-buffer--setup))
