@@ -1,12 +1,12 @@
 ;;; vertico-multiform.el --- Configure Vertico in different forms per command -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021-2025 Free Software Foundation, Inc.
+;; Copyright (C) 2021-2026 Free Software Foundation, Inc.
 
 ;; Author: Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2021
-;; Version: 2.3
-;; Package-Requires: ((emacs "28.1") (compat "30") (vertico "2.3"))
+;; Version: 2.6
+;; Package-Requires: ((emacs "29.1") (compat "30") (vertico "2.6"))
 ;; URL: https://github.com/minad/vertico
 
 ;; This file is part of GNU Emacs.
@@ -37,10 +37,11 @@
 ;;    (setq vertico-multiform-commands
 ;;          '((consult-line buffer)
 ;;            (consult-imenu reverse buffer)
-;;            (execute-extended-command flat)))
+;;            (execute-extended-command flat
+;;              (:keymap "X" execute-extended-command-for-buffer))))
 ;;
 ;;    (setq vertico-multiform-categories
-;;          '((file buffer grid)
+;;          '((file buffer grid (:keymap . vertico-directory-map))
 ;;            (imenu (:not indexed mouse))
 ;;            (symbol (vertico-sort-function . vertico-sort-alpha))))
 ;;
@@ -83,8 +84,8 @@ category settings have lower precedence than
 
 (defun vertico-multiform--toggle (arg)
   "Toggle modes from stack depending on ARG."
-  (when-let ((win (active-minibuffer-window))
-             (modes (car vertico-multiform--stack)))
+  (when-let* ((win (active-minibuffer-window))
+              (modes (car vertico-multiform--stack)))
     (when (> arg 0) (setq modes (reverse modes)))
     (with-selected-window win
       (dolist (m modes)
@@ -115,6 +116,7 @@ The keys in LIST can be symbols or regexps."
               'category))
         (exit (make-symbol "vertico-multiform--exit"))
         (depth (recursion-depth))
+        (kmaps nil)
         (modes nil))
     (fset exit (lambda ()
                  (when (= depth (recursion-depth))
@@ -126,18 +128,26 @@ The keys in LIST can be symbols or regexps."
     (dolist (x (cdr (or (vertico-multiform--lookup this-command vertico-multiform-commands)
                         (vertico-multiform--lookup cat vertico-multiform-categories))))
       (pcase x
+        (`(:keymap . ,key)
+         (push (if (keymapp key) key (apply #'define-keymap key)) kmaps))
         (`(:not . ,fs)
          (dolist (f fs)
-           (let ((sym (and (symbolp f) (intern-soft (format "vertico-%s-mode" f)))))
-             (push (cons :not (if (and sym (fboundp sym)) sym f)) modes))))
+           (let* ((sym (and (symbolp f) (intern-soft (format "vertico-%s-mode" f))))
+                  (mode (if (and sym (fboundp sym)) sym f)))
+             (when (or (not (symbolp mode)) (not (boundp mode)) (symbol-value mode))
+               (push (cons :not mode) modes)))))
         ((or (pred functionp) (pred symbolp))
-         (let ((sym (and (symbolp x) (intern-soft (format "vertico-%s-mode" x)))))
-           (push (if (and sym (fboundp sym)) sym x) modes)))
+         (let* ((sym (and (symbolp x) (intern-soft (format "vertico-%s-mode" x))))
+                (mode (if (and sym (fboundp sym)) sym x)))
+           (when (or (not (symbolp mode)) (not (boundp mode)) (not (symbol-value mode)))
+             (push mode modes))))
         (`(,k . ,v) (set (make-local-variable k) v))
         (_ (error "Invalid multiform setting %S" x))))
     (push modes vertico-multiform--stack)
     (vertico-multiform--toggle 1)
-    (vertico--setup)))
+    (vertico--setup)
+    (when kmaps
+      (use-local-map (make-composed-keymap kmaps (current-local-map))))))
 
 (defvar-keymap vertico-multiform-map
   :doc "Additional keymap activated in multiform mode.")
@@ -166,13 +176,13 @@ The keys in LIST can be symbols or regexps."
 
 (cl-defmethod vertico--advice (&context (vertico-multiform-mode (eql t)) &rest app)
   (unwind-protect
-      (progn
+      (dlet ((completion-eager-display nil)) ;; Available on Emacs 31
         (vertico-multiform--toggle -1)
         (minibuffer-with-setup-hook #'vertico-multiform--setup
           (apply app)))
     (vertico-multiform--toggle 1)))
 
-(defun vertico-multiform--temporary-mode (mode arg)
+(defun vertico-multiform--toggle-mode-1 (mode arg)
   "Enable or disable MODE temporarily in minibuffer given ARG.
 ARG can be nil, t, -1, 1 or toggle."
   (unless (minibufferp)
@@ -193,31 +203,29 @@ ARG can be nil, t, -1, 1 or toggle."
           (setcar vertico-multiform--stack (remove mode modes))
         (push not-mode (car vertico-multiform--stack))))))
 
-(defun vertico-multiform-vertical (&optional mode)
-  "Toggle to display MODE temporarily in minibuffer.
-MODE defaults to the vertical display."
-  (interactive)
+(defun vertico-multiform--toggle-mode (mode)
+  "Toggle display MODE temporarily in minibuffer."
   (let (last)
     (dolist (m vertico-multiform--display-modes)
       (when (and (boundp m) (symbol-value m))
         (setq last m)
-        (vertico-multiform--temporary-mode m -1)))
+        (vertico-multiform--toggle-mode-1 m -1)))
     (when (eq last mode)
       (setq mode vertico-multiform--display-last))
     (when mode
-      (vertico-multiform--temporary-mode mode 1))
+      (vertico-multiform--toggle-mode-1 mode 1))
     (setq vertico-multiform--display-last last)))
 
 ;; unobtrusive must come after flat
 (dolist (name '(buffer flat grid reverse unobtrusive vertical))
   (let ((toggle (intern (format "vertico-multiform-%s" name)))
-        (label (capitalize (symbol-name name))))
-    (unless (eq name 'vertical)
-      (let ((mode (intern (format "vertico-%s-mode" name))))
-        (defalias toggle
-          (lambda () (interactive) (vertico-multiform-vertical mode))
-          (format "Toggle the %s display." name))
-        (push mode vertico-multiform--display-modes)))
+        (label (capitalize (symbol-name name)))
+        (mode (and (not (eq name 'vertical)) (intern (format "vertico-%s-mode" name)))))
+    (defalias toggle
+      (lambda () (interactive) (vertico-multiform--toggle-mode mode))
+      (format "Toggle the %s display." name))
+    (when mode
+      (push mode vertico-multiform--display-modes))
     (put toggle 'completion-predicate #'vertico--command-p)
     (define-key vertico-multiform-map (vector toggle) (cons label toggle))
     (keymap-set vertico-multiform-map (format "M-%c" (aref label 0)) toggle)))
