@@ -1,12 +1,12 @@
 ;;; vertico-reverse.el --- Reverse the Vertico display -*- lexical-binding: t -*-
 
-;; Copyright (C) 2021-2025 Free Software Foundation, Inc.
+;; Copyright (C) 2021-2026 Free Software Foundation, Inc.
 
 ;; Author: Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2021
-;; Version: 2.3
-;; Package-Requires: ((emacs "28.1") (compat "30") (vertico "2.3"))
+;; Version: 2.6
+;; Package-Requires: ((emacs "29.1") (compat "30") (vertico "2.6"))
 ;; URL: https://github.com/minad/vertico
 
 ;; This file is part of GNU Emacs.
@@ -59,7 +59,7 @@
   :global t :group 'vertico
   ;; Reset overlays
   (dolist (buf (buffer-list))
-    (when-let ((ov (buffer-local-value 'vertico--candidates-ov buf)))
+    (when-let* ((ov (buffer-local-value 'vertico--candidates-ov buf)))
       (overlay-put ov 'before-string nil)))
   (cl-callf2 rassq-delete-all vertico-reverse-map minor-mode-map-alist)
   (when vertico-reverse-mode
@@ -72,8 +72,8 @@
     (setq lines (nconc (make-list (max 0 (- vertico-count (length lines))) "\n") lines)))
   (let ((string (apply #'concat lines)))
     (add-face-text-property 0 (length string) 'default 'append string)
-    (overlay-put vertico--candidates-ov 'before-string string)
-    (overlay-put vertico--candidates-ov 'after-string nil)))
+    (overlay-put vertico--candidates-ov 'before-string string))
+  (vertico--resize-window (length lines)))
 
 (provide 'vertico-reverse)
 ;;; vertico-reverse.el ends here
